#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
  /* cfloppy version 1.0. IBM Internal Use Only. */
  /* Copyright IBM Corporation, 1987*/
  /* Copies a floppy disk on /dev/rfd0 to another floppy disk, using */
  /* memory as the intermediate storage media. */
  /* syntax:  cfloppy */
  /* Will detect unformatted output media and call the format(1) */
  /* program to format the output media. */
  /* Will allow retry if the output disk is write protected or not */
  /* ready. */
  /* Copies ten tracks at a time. */
  /* Bugs:  It would be nice to allow for -f to control the device used. */
  /*        It would be nice to allow for -n to produce multiple */
  /*        copies of the input diskette for production type runs.*/
				/* Nick Simicich, NJS@WATSON, */
				/*                njs at yktvmh*/

  /* The following define variable controls the amount of data which */
  /* is read or written in one operation. */

#define RDSIZE ((unsigned int) ((unsigned int) 7680) * ((unsigned int) 10))

  /* The following define variable controls the number of units that */
  /* can be moved in one disk insertion. */

#define SIZEMULT 200

main(argc, argv)
     int argc;
     char *argv[];
{
    char *floppy, *usefloppy, *malloc();
    int in = -1, out = -1;
    register int i;
    
    off_t bytes = 0, passbytes = 0, totbytes = 0;
    
    off_t outbytes = 0;

    char response[10];
    
    
    
    if(NULL == (floppy = malloc(RDSIZE*SIZEMULT))) {
	perror("Could not get storage");
	exit(1);
	}

    do {
	printf("Insert input diskette and press enter.");
	fgets(response, sizeof(response), stdin);
	
	if (in == -1) {
	    if (-1 != out) {
		close(out);
		out = -1;
		}
	    
	    if(-1 == (in = open("/dev/rfd0", O_RDONLY))) {
		perror("Could not open the input device");
		exit(1);
		}
	    }
	if(-1 == lseek(in, totbytes, SEEK_SET)) {
	    perror("Seek failed on input file");
	    exit(1);
	    }

	/* Read until you don't get a full read */

	for(i = 0, passbytes = RDSIZE, usefloppy = floppy;
	    i<SIZEMULT && passbytes == RDSIZE; 
	    usefloppy += RDSIZE, i++, bytes += passbytes) {
	    
	    if(-1 == (passbytes = read(in, usefloppy, RDSIZE))) {
		perror("Problem with read.");
		exit(1);
		}
	    }
	
	if(-1 == out) {
	    if(in != -1) {
		close(in);
		in = -1;
		}
	    do {
		printf("Insert output diskette and press enter.");
		fgets(response, sizeof(response), stdin);
	
		if(-1 == (out = open("/dev/rfd0", O_WRONLY))) {

		    extern int errno ;

		    if(errno == ENOTREADY) {
			fprintf(stderr, "Device not ready, ");
			continue;
			}
		    
		    if(errno == EWRPROTECT) {
			fprintf(stderr, "Device write protected, ");
			continue;
			}
		    
		    if(errno == EFORMAT) {
			FILE *pipef;
			
			fprintf(stderr, "Formatting output diskette, may take several minutes\n");
			pipef = popen("format", "w");
			if (pipef == NULL) {
			    perror("could not invoke format with pipe");
			    fprintf(stderr, "Try another output diskette, ");
			    continue;
			    }

			fputs("\n", pipef);
			if(pclose(pipef)) {
			    perror("format not successful");
			    fprintf(stderr, "Try another output diskette, ");
			    continue;
			    }

			else {
			    fprintf(stderr, "format successful,  continuing..\n");
			    }

			if(-1 == (out = open("/dev/rfd0", O_WRONLY))) {
			    perror("open for output failed");
			    fprintf(stderr, "Try another output diskette, ");
			    continue;
			    }

			}

		    else {
			perror("open for output failed");
			fprintf(stderr, "Try another output diskette, ");
			continue;
			}
		    }
	        } while(out == -1); 
	    }
	if(-1 == lseek(out, totbytes, SEEK_SET)) {
	    perror("Seek failed on output file");
	    exit(1);
	    }

	for(passbytes = bytes, usefloppy = floppy;
	    passbytes > 0; 
	    usefloppy += RDSIZE, passbytes -= i, outbytes += i) {

	    i = passbytes > RDSIZE ? RDSIZE : passbytes;

	    if(-1 == write(out, usefloppy, i)) {
		perror("Problem with write");
		exit(1);
		}
	    }
	
	if(bytes != outbytes) {
	    fprintf(stderr, "Bytes read = %d, Bytes written = %d, copy not successful\n", 
		    bytes, outbytes);
	    exit(1);
	    }
	

	totbytes += bytes;
	

	} while(bytes == RDSIZE*SIZEMULT);
    if(in != -1) close(in);
    if(out != -1) close(out);
    exit(0);
    }
  
    
